// MenuWeb.cpp
/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#ifdef WIN32
#include <winsock2.h>
#else
#include <sys/time.h>
#include <unistd.h>
#endif
#include <string>
using std::string;

#include "Form.h"
#include "Cookies.h"
#include "Parse.h"

#include <vector>
#include "WebForm.h"
#include "Web.h"
#include "MenuWeb.h"


MenuWeb::MenuWeb(int style /*= MENU_STYLE_TOP*/) : Web()
,m_count(0)
,m_style(style)
,m_page(0)
,m_cookiename("page")
,m_bSetCookie(false)
,m_strClass("")
{
}


MenuWeb::MenuWeb(const string &cd,const string &cp,int style /*= MENU_STYLE_TOP*/) : Web(cd,cp)
,m_count(0)
,m_style(style)
,m_page(0)
,m_cookiename("page")
,m_bSetCookie(false)
{
}


MenuWeb::~MenuWeb()
{
	for (menuitem_v::iterator it = m_items.begin(); it != m_items.end(); it++)
	{
		MENUITEM *m = static_cast<MENUITEM *>(*it);
		delete m;
	}
}



void MenuWeb::SetMenuStyle(int style)
{
	m_style = style;
	m_bSetCookie = true;
}



const MENUITEM & MenuWeb::AddMenuItem(WebForm *form,const string &title)
{
	MENUITEM *m = new MENUITEM;

	m -> num = ++m_count;
	m -> type = MENUITEM_TYPE_FORM;
	m -> form = form;
	m -> title = title;
	m -> url = "";
	m -> src = "";
	m_items.insert(m_items.end(),m);
	return *m;
}


void MenuWeb::AddMenuSeparator()
{
	MENUITEM *m = new MENUITEM;

	m -> num = ++m_count;
	m -> type = MENUITEM_TYPE_SEPARATOR;
	m -> form = NULL;
	m -> title = "";
	m -> url = "";
	m -> src = "";
	m_items.insert(m_items.end(),m);
}


void MenuWeb::AddMenuLink(const string &url,const string &title)
{
	MENUITEM *m = new MENUITEM;

	m -> num = ++m_count;
	m -> type = MENUITEM_TYPE_URL;
	m -> form = NULL;
	m -> title = title;
	m -> url = url;
	m -> src = "";
	m_items.insert(m_items.end(),m);
}


void MenuWeb::AddMenuImage(const string &src)
{
	MENUITEM *m = new MENUITEM;

	m -> num = ++m_count;
	m -> type = MENUITEM_TYPE_URL;
	m -> form = NULL;
	m -> title = "";
	m -> url = "";
	m -> src = src;
	m_items.insert(m_items.end(),m);
}


void MenuWeb::ReadCookies()
{
	char slask[200];

	Web::ReadCookies();

	if (GetCookies() -> getvalue( (char *)m_cookiename.c_str(),slask,200))
	{
		Parse pa(slask,":");
		m_style = pa.getvalue();
		m_page = pa.getvalue();
	}
}


void MenuWeb::FormInput()
{
	char slask[200];

	Web::FormInput();

	if (GetForm() -> getvalue("page",slask,200) && *slask && atoi(slask) )
	{
		SetPage(atoi(slask));
	}
	if (GetForm() -> getvalue("style",slask,200) && *slask && atoi(slask) )
	{
		SetMenuStyle(atoi(slask));
	}
}


void MenuWeb::CreateHeader()
{
	if (m_bSetCookie)
	{
		SetTheCookie();
	}
	Web::CreateHeader();
}


void MenuWeb::GenerateDocument()
{
	int colspan = 1;
	char slask[200];
	char cls[200];

	if (m_strClass.size())
		sprintf(cls," class=%s",m_strClass.c_str());
	else
		*cls = 0;

	switch (m_style)
	{
	case MENU_STYLE_FRAME_LEFT:
		break;
	case MENU_STYLE_TOP:
		printf("<table width=100%%><tr><td>");
		// menu items
		strcpy(slask,"[ ");
		for (menuitem_v::iterator it = m_items.begin(); it != m_items.end(); it++)
		{
			MENUITEM *m = static_cast<MENUITEM *>(*it);
			
			switch (m -> type)
			{
			case MENUITEM_TYPE_FORM:
				printf("%s<a%s href=%s?page=%d>%s</a> ",
					slask,
					cls,
					GetCgiName().c_str(),
					m -> num,
					m -> title.c_str());
				strcpy(slask,"| ");
				break;
			case MENUITEM_TYPE_SEPARATOR:
				if (*slask == '|')
				{
					printf("]</td><td>");
					strcpy(slask,"[ ");
					colspan++;
				}
				break;
			case MENUITEM_TYPE_URL:
				printf("%s<a%s href=%s>%s</a> ",
					slask,
					cls,
					m -> url.c_str(),
					m -> title.c_str());
				strcpy(slask,"| ");
				break;
			case MENUITEM_TYPE_IMAGE:
				break;
			}
		}
		if (*slask != '[')
			printf("]");
		printf("</td></tr>");
		printf("</table>");
//		printf("<tr><td colspan=%d>",colspan);
		// selection
		for (menuitem_v::iterator it = m_items.begin(); it != m_items.end(); it++)
		{
			MENUITEM *m = static_cast<MENUITEM *>(*it);
			if (m -> num == m_page)
			{
				m -> form -> Display();
				break;
			}
		}
//		printf("</td></tr></table>");
		break;
	case MENU_STYLE_LEFT:
		printf("<table><tr>");
		printf("<td valign=top align=middle>");
		// menu items
		for (menuitem_v::iterator it = m_items.begin(); it != m_items.end(); it++)
		{
			MENUITEM *m = static_cast<MENUITEM *>(*it);
			
			switch (m -> type)
			{
			case MENUITEM_TYPE_FORM:
				printf("<a%s href=%s?page=%d>%s</a> ",
					cls,
					GetCgiName().c_str(),
					m -> num,
					m -> title.c_str());
				break;
			case MENUITEM_TYPE_SEPARATOR:
//				printf("<hr>");
				break;
			case MENUITEM_TYPE_URL:
				printf("<a%s href=%s>%s</a> ",
					cls,
					m -> url.c_str(),
					m -> title.c_str());
				break;
			case MENUITEM_TYPE_IMAGE:
				break;
			}
			printf("<br>");
		}
		printf("</td>");
		printf("<td valign=top>");
		// selection
		for (menuitem_v::iterator it = m_items.begin(); it != m_items.end(); it++)
		{
			MENUITEM *m = static_cast<MENUITEM *>(*it);
			if (m -> num == m_page)
			{
				m -> form -> Display();
				break;
			}
		}
		printf("</td>");
		printf("</tr></table>");
		break;
	}
}


void MenuWeb::SetTheCookie()
{
	char slask[200];
	
	sprintf(slask,"%d:%d",m_style,m_page);
	SetCookie(m_cookiename.c_str(),slask);
}


